/*
 
 Copyright (C) 2010 Timo Lassmann <timolassmann@gmail.com>
 
 This file is part of delve.
 
 Delve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 Delve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Delve.  If not, see <http://www.gnu.org/licenses/>.
 
 */

#include "interface.h"
#include <math.h>
#include "delve.h"

struct parameters* interface(struct parameters* param,int argc, char *argv[])
{
	int c;
	int i;
	int j;
	int help = 0;

	if (argc < 2){
		usage();
		exit(0);
	}
	if (argc < 3){
		if(!strcmp(argv[1],"index")){ 
			usage_index();
		}else if(!strcmp(argv[1],"seed")){
			usage_seed();
		}else if(!strcmp(argv[1],"align")){
			usage_align();
		}else if(!strcmp(argv[1],"realign")){
			usage_realign();
		}else if(!strcmp(argv[1],"post")){
			usage_post();
		}else if(!strcmp(argv[1],"sim")){
			usage_sim();
		}else if(!strcmp(argv[1],"show")){
			usage_show();
		}else if(!strcmp(argv[1],"bedsim")){
			usage_bedsim();
		}else if(!strcmp(argv[1],"eval")){
			usage_eval();
		}else if(!strcmp(argv[1],"-h")){
			usage();
		}else if(!strcmp(argv[1],"-help")){
			usage();
		}else{
			fprintf(stderr, "command '%s' not recognized\n", argv[1]);
			exit(1);
		}
		exit(0);
	}
	
	param = malloc(sizeof(struct parameters));
	param->infiles = 0;
	param->infile = 0;
	param->outfile = 0;
	param->hmm_file = 0;
	param->quiet_flag = 0;
	param->num_query = 1200000;
	param->num_threads = 4;
	param->iterations = 3;
	param->genome_window = 64;
	param->r = 12;
	param->step = 8;
	param->upper_limit  = 1;
	param->alignment_window = 10;
	param->viterbi = 0;
	param->count_mismatches_posterior = 0;
	param->train_pair_hmm = 0;
	
	param->command_line = 0;
	param->assembly_version = 0;
	param->local_weighting = 0;
	param->training_sequences = 1000000;
	
	param->sim_length = 30;
	param->sim_start_error = 0.02f;
	param->sim_end_error = 0.03f;
	param->sim_mismatch_prob = 0.8f;
	param->lightDB = 0;
	param->window_start = 0;
	param->wiggle = 0;
	param->sim_numseq = 1000000;
	param->quality_cutoff = 0.0;
	param->control_file  = 0;
	param->error_cutoff = 4;
	param->random_model_prior = 0.1f;
	param->unaligned_probability_cutoff = 0.5;
	param->fdr = 0.05;
	param->mean_error = 0.0f;
	while (1){	 
		static struct option long_options[] ={
			{"as",required_argument,0,OPT_ASSEMBLY},
			{"viterbi",0,0,OPT_VITERBI},
			{"ts",required_argument,0,OPT_TRAINING_SEQUENCES},
			{"read-len",required_argument,0,OPT_READLEN},
			{"ranp",required_argument,0,OPT_RANP},
			{"start-error",required_argument,0,OPT_AT_START_ERROR},
			{"stop-error",required_argument,0,OPT_AT_END_ERROR},
			{"mismatch-rate",required_argument,0,OPT_INDEL},
			{"sw",required_argument,0,OPT_WINDOW_START},
			{"wig",0,0,OPT_WIGGLE},
			{"count-mismatches",0,0,OPT_COUNT_MISMATCHES},
			{"lw",required_argument,0,OPT_LOCAL_WEIGHT},
			{"cut",required_argument,0,OPT_FDR},
			{"quiet",required_argument,0,'q'},
			{"help",0,0,'h'},
			{"num",required_argument,0,'n'},
			{0, 0, 0, 0}
		};
		
		int option_index = 0;
		c = getopt_long_only (argc, argv,"o:qhu:i:t:l:s:m:n:c:k:",long_options, &option_index);
		
		if (c == -1){
			break;
		}
		
		switch(c) {
			case 0:
				
				break;
			case OPT_TRAINING_SEQUENCES:
				param->training_sequences = atoi(optarg);
				break;
			case OPT_LOCAL_WEIGHT:
				param->local_weighting = atoi(optarg);
				break;
			case OPT_VITERBI:
				param->viterbi = 1;
				break;
			case OPT_ASSEMBLY:
				param->assembly_version = optarg;
				break;
			case OPT_READLEN:
				param->sim_length = atoi(optarg);
				break;
			case OPT_RANP:
				param->random_model_prior = atof(optarg);
				break;
			case OPT_AT_START_ERROR:
				param->sim_start_error = atof(optarg);
				break;
			case OPT_AT_END_ERROR:
				param->sim_end_error = atof(optarg);
				break;
			case OPT_INDEL:
				param->sim_mismatch_prob = atof(optarg);
				break;
			case OPT_WINDOW_START:
				param->window_start = atoi(optarg);
				break;
			case OPT_WIGGLE:
				param->wiggle = 1;
				break;
			case OPT_COUNT_MISMATCHES:
				param->count_mismatches_posterior = 1;
				break;
				
			case OPT_FDR:
				param->fdr = atof(optarg);
				break;
				
			case 's':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -s requires an argument\n");
					exit(-1);
				}
				param->step = atoi(optarg);
				break;
				
				
			case 'n':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -n requires an argument\n");
					exit(-1);
				}
				param->sim_numseq = atoi(optarg);
				break;
				
			case 'k':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -n requires an argument\n");
					exit(-1);
				}
				param->error_cutoff = atoi(optarg);
				break;
			case 'l':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -l requires an argument\n");
					exit(-1);
				}
				param->r = atoi(optarg);
				break;
				
			case 'u':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -u requires an argument\n");
					exit(-1);
				}
				param->upper_limit = atoi(optarg);
				break;
			case 't':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -t requires an argument\n");
					exit(-1);
				}
				param->num_threads = atoi(optarg);
				break;
				
			case 'i':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -i requires an argument\n");
					exit(-1);
				}
				param->iterations = atoi(optarg);
				break;
			case 'o':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -o requires an argument\n");
					exit(-1);
				}
				param->outfile = optarg;
				break;	
			case 'c':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -c requires an argument\n");
					exit(-1);
				}
				param->control_file = optarg;
				break;	
			case 'm':
				if(optarg[0] == '-'){
					fprintf(stderr,"option -m requires an argument\n");
					exit(-1);
				}
				param->hmm_file = optarg;
				break;	
			case 'q':
				param->quality_cutoff =  1.0 - pow(10.0, -1.0 *   atof(optarg) / 10.0); 
				break;
			case 'h':
				help = 1;
				break;
			case '?':
				exit(1);
				break;
			default:
				fprintf(stderr,"default\n\n\n\n");
				abort ();
		}
	}
	//fprintf(stderr,"Viterbi: %d\n",param->viterbi);
	
	if(help){
		usage();
		free(param);
		exit(0);
	}
	
	
	
	/*if (!isatty(0)){
		j = 1;
	}else{
		j = 0;
	}*/
	param->infile = malloc(sizeof(char*)* (argc-optind ));
	
	c = 0;
	while (optind < argc){
		param->infile[c] =  argv[optind++];
		c++;
		/*if(c ==1 && j == 1){
			param->infile[c] = 0;
			c++;
		}*/
	}
	param->infiles = c;
	
	param->command_line = malloc(sizeof(char) * MAX_LINE);
	c = 0;
	for(i =0 ; i < argc;i++){
		for(j = 0; j < strlen(argv[i]);j++){
			param->command_line[c] = argv[i][j];
			c++;
		}
		param->command_line[c] = ' ';
		c++;
		
	}
	param->command_line[c] = 0;
	return param;
}


void free_param(struct parameters* param)
{
	free(param->infile);
	free(param->command_line);
	free(param);
}

void usage()
{
        fprintf(stdout, "\n%s, Copyright (C) 2010 Timo Lassmann <timolassmann@gmail.com>\n", VERSION);
	fprintf(stdout, "\n");
	fprintf(stdout, "Usage:   delve <command> [options]\n\n");
        fprintf(stdout, "Command: index         index sequences in the FASTA format\n");
        fprintf(stdout, "         seed          seed alignment \n");
        fprintf(stdout, "         align         build pHMM model and score alignments\n");
	fprintf(stdout, "         post          generate BEDgraph files\n");
	fprintf(stdout, "         realign       realign reads in SAM file\n");
	fprintf(stdout, "         show          prints model parameters\n");
	fprintf(stdout, "\n");
	
	fprintf(stdout, "         sim           modifies sequences simulated by wgsim\n");
	fprintf(stdout, "         bedsim        simulates reads in BED defined regions\n");
	fprintf(stdout, "         eval          evaluates simulated read mappings\n");
	
        fprintf(stdout, "\n");
}

void usage_index()
{
        fprintf(stdout, "\n%s, Copyright (C) 2010 Timo Lassmann <timolassmann@gmail.com>\n", VERSION);
	fprintf(stdout, "\n");
	fprintf(stdout, "Usage:   delve index <in.fasta>\n\n");
        fprintf(stdout, "\n");
}

void usage_seed()
{
        fprintf(stdout, "\n%s, Copyright (C) 2010 Timo Lassmann <timolassmann@gmail.com>\n", VERSION);
	fprintf(stdout, "\n");
	fprintf(stdout, "Usage:   delve seed [options] <in.fastq> <reference.fa>\n\n");
	fprintf(stdout, "Options: -l INT     seed length [12]\n");
        fprintf(stdout, "         -s INT     step size [8]\n");
        fprintf(stdout, "         -t INT     number of threads [4]\n");
	fprintf(stdout, "         -o STR     SAM output file [STDOUT]\n");
	fprintf(stdout, "         -as STR    adds assembly infomation to SAM header [NA]\n");
	
        fprintf(stdout, "\n");
}

void usage_align()
{
        fprintf(stdout, "\n%s, Copyright (C) 2010 Timo Lassmann <timolassmann@gmail.com>\n", VERSION);
	fprintf(stdout, "\n");
	fprintf(stdout, "Usage:   delve align [options] <in.sam> <reference.fa>\n\n");
	fprintf(stdout, "Options: -t    INT  number of threads [4]\n");
	fprintf(stdout, "         -o    STR  SAM output file [STDOUT]\n");
	fprintf(stdout, "         -u    INT  number of reported alignments [1]\n");
	fprintf(stdout, "         -m    STR  model output file [NA]\n");
	fprintf(stdout, "         -ts   INT  minimum number of reads to train model (in millions) [1]\n");
	fprintf(stdout, "         -lw   NA   use local density of mappings score alignments[off]\n");
	fprintf(stdout, "         -as   STR  adds assembly infomation to SAM header [NA]\n");
	fprintf(stdout, "         -viterbi   use viterbi decoding to score alignments [off]\n");
        fprintf(stdout, "\n");
}

void usage_realign()
{
        fprintf(stdout, "\n%s, Copyright (C) 2010 Timo Lassmann <timolassmann@gmail.com>\n", VERSION);
	fprintf(stdout, "\n");
	fprintf(stdout, "Usage:   delve realign [options] <in.sam> <reference.fa>\n\n");
	fprintf(stdout, "Options: -o STR     SAM output file [STDOUT]\n");
	fprintf(stdout, "         -u INT     number of reported alignments [1]\n");
	fprintf(stdout, "         -as STR    adds assembly infomation to SAM header [NA]\n");
        fprintf(stdout, "\n");
}

void usage_post()
{
        fprintf(stdout, "\n%s, Copyright (C) 2010 Timo Lassmann <timolassmann@gmail.com>\n", VERSION);
	fprintf(stdout, "\n");
	fprintf(stdout, "Usage:   delve post [options] -m <model> <in.sam> <reference.fa>\n\n");
	fprintf(stdout, "Options: -o   STR     BEDgraph output file [STDOUT]\n");
	fprintf(stdout, "         -q   INT     mapping quality cutoff [0]\n");
	fprintf(stdout, "         -m   STR     model output file [required]\n");
	fprintf(stdout, "         -wig NA      use WIG format instead of BEDgraph.\n");
	
        fprintf(stdout, "\n");
}

void usage_sim()
{
        fprintf(stdout, "\n%s, Copyright (C) 2010 Timo Lassmann <timolassmann@gmail.com>\n", VERSION);
	fprintf(stdout, "\n");
	fprintf(stdout, "Usage:   delve sim [options] <in.fastq> FLOAT FLOAT FLOAT FLOAT FLOAT\n\n");
	fprintf(stdout, "Options: -o STR     fastq output file [STDOUT]\n");
        fprintf(stdout, "\n");
	fprintf(stdout, "The FLOATs correspond to:\n");
	fprintf(stdout, "     1 - error rate at 5' end [0.01]\n");
	fprintf(stdout, "     2 - error rate at 3' end [0.03]\n");
	fprintf(stdout, "     3 - mismatch rate [0.8]\n");
	fprintf(stdout, "     4 - probability of 5' G addition [0.0]\n");
	fprintf(stdout, "     5 - probability of 3' A addition [0.0]\n");
	
}

void usage_bedsim()
{
        fprintf(stdout, "\n%s, Copyright (C) 2010 Timo Lassmann <timolassmann@gmail.com>\n", VERSION);
	fprintf(stdout, "\n");
	fprintf(stdout, "Usage:   delve bedsim [options] <in.bed> <reference.fa>\n\n");
	fprintf(stdout, "Options: -read-len      INT    Length of simulated reads [30]\n");
	fprintf(stdout, "         -start-error   FLT    error rate at 5' end [0.02]\n");
	fprintf(stdout, "         -stop-error    FLT    error rate at 3' end [0.03]\n");
	fprintf(stdout, "         -mismatch-rate FLT    fraction of simulated mismatches [0.8]\n");
	fprintf(stdout, "         -sw            INT    simulate reads from +- INT start of feature [0]\n");
	fprintf(stdout, "         -n             INT    number of reads to simulate [1000000]\n");
	fprintf(stdout, "         -c             STR    name of control fasta file [NA]\n");
        fprintf(stdout, "\n");
}


void usage_eval()
{
        fprintf(stdout, "\n%s, Copyright (C) 2010 Timo Lassmann <timolassmann@gmail.com>\n", VERSION);
	fprintf(stdout, "\n");
	fprintf(stdout, "Usage:   delve eval <in.sam>\n\n");
        fprintf(stdout, "\n");
}



void usage_show()
{
        fprintf(stdout, "\n%s, Copyright (C) 2010 Timo Lassmann <timolassmann@gmail.com>\n", VERSION);
	fprintf(stdout, "\n");
	fprintf(stdout, "Usage:   delve show <in.model>\n\n");
	
}


